@@ -7,14 +7,12 @@
 
 import argparse
 import logging
-import os
 import sys
 import tempfile
 import threading
 
-from benchexec.cgroups import CPUACCT, CPUSET, FREEZER, MEMORY, find_my_cgroups
+from benchexec.cgroups import Cgroups
 from benchexec.runexecutor import RunExecutor
-from benchexec import util
 
 sys.dont_write_bytecode = True  # prevent creation of .pyc files
 
@@ -34,22 +32,36 @@ def check_cgroup_availability(wait=1):
     my_cgroups = runexecutor.cgroups
 
     if not (
-        CPUACCT in my_cgroups
-        and CPUSET in my_cgroups
+        my_cgroups.CPU in my_cgroups
         # and FREEZER in my_cgroups # For now, we do not require freezer
-        and MEMORY in my_cgroups
+        and my_cgroups.MEMORY in my_cgroups
     ):
         sys.exit(1)
 
+    if my_cgroups.CPUSET in my_cgroups:
+        cores = my_cgroups.read_allowed_cpus()
+        mems = my_cgroups.read_allowed_memory_banks()
+    else:
+        # Use dummy value (does not matter which) to let execute_run() fail.
+        cores = [0]
+        mems = [0]
+
     with tempfile.NamedTemporaryFile(mode="rt") as tmp:
-        runexecutor.execute_run(
+        execution = runexecutor.execute_run(
             ["sh", "-c", f"sleep {wait}; cat /proc/self/cgroup"],
             tmp.name,
-            memlimit=1024 * 1024,  # set memlimit to force check for swapaccount
+            memlimit=100 * 1024 * 1024,  # set memlimit to force check for swapaccount
             # set cores and memory_nodes to force usage of CPUSET
-            cores=util.parse_int_list(my_cgroups.get_value(CPUSET, "cpus")),
-            memory_nodes=my_cgroups.read_allowed_memory_banks(),
+            cores=cores,
+            memory_nodes=mems,
         )
+        if "terminationreason" in execution:
+            logging.error(
+                "Cgroup check terminated with reason '%s'.",
+                execution["terminationreason"],
+            )
+            sys.exit(1)
+        assert execution["exitcode"].raw == 0, execution
         lines = []
         for line in tmp:
             line = line.strip()
@@ -59,14 +71,15 @@ def check_cgroup_availability(wait=1):
                 and not all(c == "-" for c in line)
             ):
                 lines.append(line)
-    task_cgroups = find_my_cgroups(lines, fallback=False)
+    task_cgroups = Cgroups.from_system(cgroup_procinfo=lines)
 
     fail = False
-    for subsystem in CPUACCT, CPUSET, MEMORY, FREEZER:
+    expected_subsystems = [my_cgroups.FREEZE]
+    if my_cgroups.version == 1:
+        expected_subsystems += [my_cgroups.CPU, my_cgroups.CPUSET, my_cgroups.MEMORY]
+    for subsystem in expected_subsystems:
         if subsystem in my_cgroups:
-            if not task_cgroups[subsystem].startswith(
-                os.path.join(my_cgroups[subsystem], "benchmark_")
-            ):
+            if not str(task_cgroups[subsystem]).startswith(str(my_cgroups[subsystem])):
                 logging.warning(
                     "Task was in cgroup %s for subsystem %s, "
                     "which is not the expected sub-cgroup of %s. "
@@ -89,6 +102,7 @@ def check_cgroup_availability_in_thread(options):
     (and this will happen if "benchexec -N" is used).
     """
     thread = _CheckCgroupsThread(options)
+    thread.daemon = True
     thread.start()
     thread.join()
     if thread.error:
