@@ -14,6 +14,9 @@ from benchexec import result
 from benchexec.tablegenerator import util
 from benchexec.tablegenerator.columns import ColumnType
 
+# It's important to make sure on *all* entry points / methods which perform arithmetics that the correct
+# rounding / context is used.
+DECIMAL_CONTEXT = decimal.Context(rounding=decimal.ROUND_HALF_UP)
 
 nan = Decimal("nan")
 inf = Decimal("inf")
@@ -65,55 +68,56 @@ class StatValue(object):
 
     @classmethod
     def from_list(cls, values):
-        if any(v is not None and v.is_nan() for v in values):
-            return StatValue(nan, nan, nan, nan, nan, nan)
+        with decimal.localcontext(DECIMAL_CONTEXT):
+            if any(v is not None and v.is_nan() for v in values):
+                return StatValue(nan, nan, nan, nan, nan, nan)
 
-        values = sorted(v for v in values if v is not None)
-        if not values:
-            return None
+            values = sorted(v for v in values if v is not None)
+            if not values:
+                return None
 
-        values_len = len(values)
-        min_value = values[0]
-        max_value = values[-1]
+            values_len = len(values)
+            min_value = values[0]
+            max_value = values[-1]
 
-        if min_value == -inf and max_value == +inf:
-            values_sum = nan
-            mean = nan
-            stdev = nan
-        elif max_value == inf:
-            values_sum = inf
-            mean = inf
-            stdev = inf
-        elif min_value == -inf:
-            values_sum = -inf
-            mean = -inf
-            stdev = inf
-        else:
-            values_sum = sum(values)
-            mean = values_sum / values_len
+            if min_value == -inf and max_value == +inf:
+                values_sum = nan
+                mean = nan
+                stdev = nan
+            elif max_value == inf:
+                values_sum = inf
+                mean = inf
+                stdev = inf
+            elif min_value == -inf:
+                values_sum = -inf
+                mean = -inf
+                stdev = inf
+            else:
+                values_sum = sum(values)
+                mean = values_sum / values_len
 
-            # The scaling is just to avoid having too few decimal digits when printing,
-            # the value is still just 0.
-            stdev = Decimal(0).scaleb(-decimal.getcontext().prec)
-            for v in values:
-                diff = v - mean
-                stdev += diff * diff
-            stdev = (stdev / values_len).sqrt()
+                # The scaling is just to avoid having too few decimal digits when printing,
+                # the value is still just 0.
+                stdev = Decimal(0).scaleb(-decimal.getcontext().prec)
+                for v in values:
+                    diff = v - mean
+                    stdev += diff * diff
+                stdev = (stdev / values_len).sqrt()
 
-        half, len_is_odd = divmod(values_len, 2)
-        if len_is_odd:
-            median = values[half]
-        else:
-            median = (values[half - 1] + values[half]) / Decimal(2)
+            half, len_is_odd = divmod(values_len, 2)
+            if len_is_odd:
+                median = values[half]
+            else:
+                median = (values[half - 1] + values[half]) / Decimal(2)
 
-        return StatValue(
-            values_sum,
-            min=min_value,
-            max=max_value,
-            avg=mean,
-            median=median,
-            stdev=stdev,
-        )
+            return StatValue(
+                values_sum,
+                min=min_value,
+                max=max_value,
+                avg=mean,
+                median=median,
+                stdev=stdev,
+            )
 
 
 def get_stats_of_run_set(runResults, correct_only):
@@ -128,19 +132,18 @@ def get_stats_of_run_set(runResults, correct_only):
     stats = []
     for index, column in enumerate(columns):
         col_type = column.type.type
-        if col_type != ColumnType.text:
-            if col_type == ColumnType.status:
-                column_stats = _get_stats_of_status_column(runResults, index)
+        if col_type == ColumnType.status:
+            column_stats = _get_stats_of_status_column(runResults, index)
 
-            else:
-                assert column.is_numeric()
-                values = (run_result.values[index] for run_result in runResults)
-                column_stats = _get_stats_of_number_column(
-                    values, status_list, correct_only
-                )
+        elif col_type == ColumnType.text:
+            column_stats = None
 
         else:
-            column_stats = None
+            assert column.is_numeric()
+            values = (run_result.values[index] for run_result in runResults)
+            column_stats = _get_stats_of_number_column(
+                values, status_list, correct_only
+            )
 
         stats.append(column_stats)
 
@@ -202,16 +205,19 @@ def _get_stats_of_number_column(values, categoryList, correct_only):
     return stats
 
 
-def _get_stats_of_status_column(run_results, col):
+def _get_stats_of_status_column(run_results, col_index):
     stats = ColumnStatistics()
     stats.score = StatValue(sum(run_result.score or 0 for run_result in run_results))
 
     stats.total = StatValue(
-        sum(1 for run_result in run_results if run_result.values[col])
+        sum(1 for run_result in run_results if run_result.values[col_index])
     )
 
     counts = collections.Counter(
-        (run_result.category, result.get_result_classification(run_result.values[col]))
+        (
+            run_result.category,
+            result.get_result_classification(run_result.values[col_index]),
+        )
         for run_result in run_results
     )
 
