@@ -126,6 +126,12 @@ class ColumnsTest(unittest.TestCase):
         )
         self.assertEqual(formatted_value, "0.00000000001")
 
+    def test_format_value_many_digits(self):
+        formatted_value_no_align_rounded = self.measure_column.format_value(
+            "0.09999999999999999", *self.default_optionals
+        )
+        self.assertEqual(formatted_value_no_align_rounded, "0.1000")
+
     def test_format_value_align_decimal(self):
         formatted_value_aligned = self.measure_column.format_value(
             "1.555s", "html_cell"
@@ -176,6 +182,15 @@ class ColumnsTest(unittest.TestCase):
         formatted_value_aligned = small_value_column.format_value("2", "html_cell")
         self.assertEqual(formatted_value_aligned, ".0000000002&#x2007;&#x2007;")
 
+    def test_invalid_rounding_mode(self):
+        import decimal
+
+        decimal.getcontext().rounding = decimal.ROUND_HALF_DOWN
+        formatted_value_no_align_zeros_cut = self.measure_column.format_value(
+            "5.7715", *self.default_optionals
+        )
+        self.assertEqual(formatted_value_no_align_zeros_cut, "5.772")
+
     def test_format_value_align_int(self):
         formatted_value_int_aligned = self.measure_column.format_value(
             "20", "html_cell"
