@@ -10,7 +10,6 @@ import logging
 import os
 import re
 import subprocess
-import sys
 import tempfile
 import threading
 import time
@@ -21,11 +20,10 @@ from benchexec import container
 from benchexec import containerexecutor
 from benchexec import filehierarchylimit
 from benchexec.runexecutor import RunExecutor
+from benchexec.cgroups import Cgroups
 from benchexec import runexecutor
 from benchexec import util
 
-sys.dont_write_bytecode = True  # prevent creation of .pyc files
-
 here = os.path.dirname(__file__)
 base_dir = os.path.join(here, "..")
 bin_dir = os.path.join(base_dir, "bin")
@@ -37,12 +35,17 @@ trivial_run_grace_time = 0.2
 class TestRunExecutor(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
-        cls.longMessage = True
-        cls.maxDiff = None
-        logging.disable(logging.NOTSET)  # need to make sure to get all messages
         if not hasattr(cls, "assertRegex"):
             cls.assertRegex = cls.assertRegexpMatches
 
+        cls.cgroups = Cgroups.initialize()
+
+        cls.echo = shutil.which("echo") or "/bin/echo"
+        cls.sleep = shutil.which("sleep") or "/bin/sleep"
+        cls.cat = shutil.which("cat") or "/bin/cat"
+        cls.dd = shutil.which("dd") or "/bin/dd"
+        cls.grep = shutil.which("grep") or "/bin/grep"
+
     def setUp(self, *args, **kwargs):
         with self.skip_if_logs(
             "Cannot reliably kill sub-processes without freezer cgroup"
@@ -150,6 +153,9 @@ class TestRunExecutor(unittest.TestCase):
             "blkio-read",
             "blkio-write",
             "starttime",
+            "pressure-cpu-some",
+            "pressure-io-some",
+            "pressure-memory-some",
         }
         expected_keys.update(additional_keys)
         for key in result.keys():
@@ -183,17 +189,17 @@ class TestRunExecutor(unittest.TestCase):
             self.assertEqual(int(result["exitsignal"]), exitcode.signal, msg)
 
     def test_command_output(self):
-        if not os.path.exists("/bin/echo"):
-            self.skipTest("missing /bin/echo")
-        (_, output) = self.execute_run("/bin/echo", "TEST_TOKEN")
-        self.check_command_in_output(output, "/bin/echo TEST_TOKEN")
+        if not os.path.exists(self.echo):
+            self.skipTest("missing echo")
+        (_, output) = self.execute_run(self.echo, "TEST_TOKEN")
+        self.check_command_in_output(output, f"{self.echo} TEST_TOKEN")
         self.assertEqual(output[-1], "TEST_TOKEN", "run output misses command output")
         for line in output[1:-1]:
             self.assertRegex(line, "^-*$", "unexpected text in run output")
 
     def test_command_error_output(self):
-        if not os.path.exists("/bin/echo"):
-            self.skipTest("missing /bin/echo")
+        if not os.path.exists(self.echo):
+            self.skipTest("missing echo")
         if not os.path.exists("/bin/sh"):
             self.skipTest("missing /bin/sh")
 
@@ -210,7 +216,7 @@ class TestRunExecutor(unittest.TestCase):
                 os.remove(error_filename)
 
         (output_lines, error_lines) = execute_Run_intern(
-            "/bin/sh", "-c", "/bin/echo ERROR_TOKEN >&2"
+            "/bin/sh", "-c", f"{self.echo} ERROR_TOKEN >&2"
         )
         self.assertEqual(
             error_lines[-1], "ERROR_TOKEN", "run error output misses command output"
@@ -220,9 +226,9 @@ class TestRunExecutor(unittest.TestCase):
         for line in error_lines[1:-1]:
             self.assertRegex(line, "^-*$", "unexpected text in run error output")
 
-        (output_lines, error_lines) = execute_Run_intern("/bin/echo", "OUT_TOKEN")
-        self.check_command_in_output(output_lines, "/bin/echo OUT_TOKEN")
-        self.check_command_in_output(error_lines, "/bin/echo OUT_TOKEN")
+        (output_lines, error_lines) = execute_Run_intern(self.echo, "OUT_TOKEN")
+        self.check_command_in_output(output_lines, f"{self.echo} OUT_TOKEN")
+        self.check_command_in_output(error_lines, f"{self.echo} OUT_TOKEN")
         self.assertEqual(
             output_lines[-1], "OUT_TOKEN", "run output misses command output"
         )
@@ -232,22 +238,22 @@ class TestRunExecutor(unittest.TestCase):
             self.assertRegex(line, "^-*$", "unexpected text in run error output")
 
     def test_command_result(self):
-        if not os.path.exists("/bin/echo"):
-            self.skipTest("missing /bin/echo")
-        (result, _) = self.execute_run("/bin/echo", "TEST_TOKEN")
-        self.check_exitcode(result, 0, "exit code of /bin/echo is not zero")
+        if not os.path.exists(self.echo):
+            self.skipTest("missing echo")
+        (result, _) = self.execute_run(self.echo, "TEST_TOKEN")
+        self.check_exitcode(result, 0, "exit code of echo is not zero")
         self.assertAlmostEqual(
             result["walltime"],
             trivial_run_grace_time,
             delta=trivial_run_grace_time,
-            msg="walltime of /bin/echo not as expected",
+            msg="walltime of echo not as expected",
         )
         if "cputime" in result:  # not present without cpuacct cgroup
             self.assertAlmostEqual(
                 result["cputime"],
                 trivial_run_grace_time,
                 delta=trivial_run_grace_time,
-                msg="cputime of /bin/echo not as expected",
+                msg="cputime of echo not as expected",
             )
         self.check_result_keys(result)
 
@@ -320,10 +326,10 @@ class TestRunExecutor(unittest.TestCase):
             self.assertRegex(line, "^-*$", "unexpected text in run output")
 
     def test_walltime_limit(self):
-        if not os.path.exists("/bin/sleep"):
-            self.skipTest("missing /bin/sleep")
+        if not os.path.exists(self.sleep):
+            self.skipTest("missing sleep")
         (result, output) = self.execute_run(
-            "/bin/sleep", "10", walltimelimit=1, expect_terminationreason="walltime"
+            self.sleep, "10", walltimelimit=1, expect_terminationreason="walltime"
         )
 
         self.check_exitcode(result, 9, "exit code of killed process is not 9")
@@ -338,10 +344,10 @@ class TestRunExecutor(unittest.TestCase):
                 result["cputime"],
                 trivial_run_grace_time,
                 delta=trivial_run_grace_time,
-                msg="cputime of /bin/sleep is not approximately zero",
+                msg="cputime of sleep is not approximately zero",
             )
 
-        self.check_command_in_output(output, "/bin/sleep 10")
+        self.check_command_in_output(output, f"{self.sleep} 10")
         for line in output[1:]:
             self.assertRegex(line, "^-*$", "unexpected text in run output")
 
@@ -406,64 +412,92 @@ class TestRunExecutor(unittest.TestCase):
         for line in output[1:]:
             self.assertRegex(line, "^-*$", "unexpected text in run output")
 
+    def test_memory_limit(self):
+        if not os.path.exists(self.dd):
+            self.skipTest("missing dd")
+        memlimit = 100_000_000
+        (result, output) = self.execute_run(
+            self.dd,
+            "if=/dev/zero",
+            "of=/dev/null",
+            f"bs={memlimit}",
+            "count=1",
+            memlimit=memlimit,
+            expect_terminationreason="memory",
+        )
+
+        self.check_exitcode(result, 9, "exit code of killed process is not 9")
+        self.assertAlmostEqual(
+            result["memory"],
+            memlimit,
+            delta=memlimit // 100,
+            msg="memory is not approximately the amount after which the process should have been killed",
+        )
+
+        self.check_command_in_output(
+            output, f"{self.dd} if=/dev/zero of=/dev/null bs={memlimit} count=1"
+        )
+        for line in output[1:]:
+            self.assertRegex(line, "^-*$", "unexpected text in run output")
+
     def test_input_is_redirected_from_devnull(self):
-        if not os.path.exists("/bin/cat"):
-            self.skipTest("missing /bin/cat")
-        (result, output) = self.execute_run("/bin/cat", walltimelimit=1)
+        if not os.path.exists(self.cat):
+            self.skipTest("missing cat")
+        (result, output) = self.execute_run(self.cat, walltimelimit=1)
 
         self.check_exitcode(result, 0, "exit code of process is not 0")
         self.assertAlmostEqual(
             result["walltime"],
             trivial_run_grace_time,
             delta=trivial_run_grace_time,
-            msg='walltime of "/bin/cat < /dev/null" is not approximately zero',
+            msg='walltime of "cat < /dev/null" is not approximately zero',
         )
         if "cputime" in result:  # not present without cpuacct cgroup
             self.assertAlmostEqual(
                 result["cputime"],
                 trivial_run_grace_time,
                 delta=trivial_run_grace_time,
-                msg='cputime of "/bin/cat < /dev/null" is not approximately zero',
+                msg='cputime of "cat < /dev/null" is not approximately zero',
             )
         self.check_result_keys(result)
 
-        self.check_command_in_output(output, "/bin/cat")
+        self.check_command_in_output(output, self.cat)
         for line in output[1:]:
             self.assertRegex(line, "^-*$", "unexpected text in run output")
 
     def test_input_is_redirected_from_file(self):
-        if not os.path.exists("/bin/cat"):
-            self.skipTest("missing /bin/cat")
+        if not os.path.exists(self.cat):
+            self.skipTest("missing cat")
         with tempfile.TemporaryFile() as tmp:
             tmp.write(b"TEST_TOKEN")
             tmp.flush()
             tmp.seek(0)
-            (result, output) = self.execute_run("/bin/cat", stdin=tmp, walltimelimit=1)
+            (result, output) = self.execute_run(self.cat, stdin=tmp, walltimelimit=1)
 
         self.check_exitcode(result, 0, "exit code of process is not 0")
         self.assertAlmostEqual(
             result["walltime"],
             trivial_run_grace_time,
             delta=trivial_run_grace_time,
-            msg='walltime of "/bin/cat < /dev/null" is not approximately zero',
+            msg='walltime of "cat < /dev/null" is not approximately zero',
         )
         if "cputime" in result:  # not present without cpuacct cgroup
             self.assertAlmostEqual(
                 result["cputime"],
                 trivial_run_grace_time,
                 delta=trivial_run_grace_time,
-                msg='cputime of "/bin/cat < /dev/null" is not approximately zero',
+                msg='cputime of "cat < /dev/null" is not approximately zero',
             )
         self.check_result_keys(result)
 
-        self.check_command_in_output(output, "/bin/cat")
+        self.check_command_in_output(output, self.cat)
         self.assertEqual(output[-1], "TEST_TOKEN", "run output misses command output")
         for line in output[1:-1]:
             self.assertRegex(line, "^-*$", "unexpected text in run output")
 
     def test_input_is_redirected_from_stdin(self):
-        if not os.path.exists("/bin/cat"):
-            self.skipTest("missing /bin/cat")
+        if not os.path.exists(self.cat):
+            self.skipTest("missing cat")
 
         (output_fd, output_filename) = tempfile.mkstemp(".log", "output_", text=True)
         cmd = self.get_runexec_cmdline(
@@ -471,7 +505,7 @@ class TestRunExecutor(unittest.TestCase):
             "-",
             "--walltime",
             "1",
-            "/bin/cat",
+            self.cat,
             output_filename=output_filename,
         )
         try:
@@ -510,18 +544,18 @@ class TestRunExecutor(unittest.TestCase):
             float(result["walltime"].rstrip("s")),
             trivial_run_grace_time,
             delta=trivial_run_grace_time,
-            msg='walltime of "/bin/cat < /dev/null" is not approximately zero',
+            msg='walltime of "cat < /dev/null" is not approximately zero',
         )
         if "cputime" in result:  # not present without cpuacct cgroup
             self.assertAlmostEqual(
                 float(result["cputime"].rstrip("s")),
                 trivial_run_grace_time,
                 delta=trivial_run_grace_time,
-                msg='cputime of "/bin/cat < /dev/null" is not approximately zero',
+                msg='cputime of "cat < /dev/null" is not approximately zero',
             )
         self.check_result_keys(result, "returnvalue")
 
-        self.check_command_in_output(output, "/bin/cat")
+        self.check_command_in_output(output, self.cat)
         self.assertEqual(output[-1], "TEST_TOKEN", "run output misses command output")
         for line in output[1:-1]:
             self.assertRegex(line, "^-*$", "unexpected text in run output")
@@ -548,12 +582,12 @@ class TestRunExecutor(unittest.TestCase):
         self.assertEqual(output[-1], "/usr/bin")
 
     def test_stop_run(self):
-        if not os.path.exists("/bin/sleep"):
-            self.skipTest("missing /bin/sleep")
+        if not os.path.exists(self.sleep):
+            self.skipTest("missing sleep")
         thread = _StopRunThread(1, self.runexecutor)
         thread.start()
         (result, output) = self.execute_run(
-            "/bin/sleep", "10", expect_terminationreason="killed"
+            self.sleep, "10", expect_terminationreason="killed"
         )
         thread.join()
 
@@ -569,10 +603,10 @@ class TestRunExecutor(unittest.TestCase):
                 result["cputime"],
                 trivial_run_grace_time,
                 delta=trivial_run_grace_time,
-                msg="cputime of /bin/sleep is not approximately zero",
+                msg="cputime of sleep is not approximately zero",
             )
 
-        self.check_command_in_output(output, "/bin/sleep 10")
+        self.check_command_in_output(output, f"{self.sleep} 10")
         for line in output[1:]:
             self.assertRegex(line, "^-*$", "unexpected text in run output")
 
@@ -644,13 +678,13 @@ class TestRunExecutor(unittest.TestCase):
         )
 
     def test_integration(self):
-        if not os.path.exists("/bin/echo"):
-            self.skipTest("missing /bin/echo")
-        (result, output) = self.execute_run_extern("/bin/echo", "TEST_TOKEN")
-        self.check_exitcode_extern(result, 0, "exit code of /bin/echo is not zero")
+        if not os.path.exists(self.echo):
+            self.skipTest("missing echo")
+        (result, output) = self.execute_run_extern(self.echo, "TEST_TOKEN")
+        self.check_exitcode_extern(result, 0, "exit code of echo is not zero")
         self.check_result_keys(result, "returnvalue")
 
-        self.check_command_in_output(output, "/bin/echo TEST_TOKEN")
+        self.check_command_in_output(output, f"{self.echo} TEST_TOKEN")
         self.assertEqual(output[-1], "TEST_TOKEN", "run output misses command output")
         for line in output[1:-1]:
             self.assertRegex(line, "^-*$", "unexpected text in run output")
@@ -726,32 +760,39 @@ class TestRunExecutor(unittest.TestCase):
             self.setUp(additional_cgroup_subsystems=["cpu"])
         except SystemExit as e:
             self.skipTest(e)
-        if not os.path.exists("/bin/cat"):
-            self.skipTest("missing /bin/cat")
-        (result, output) = self.execute_run("/bin/cat", "/proc/self/cgroup")
-        self.check_exitcode(result, 0, "exit code of /bin/cat is not zero")
+        if not os.path.exists(self.cat):
+            self.skipTest("missing cat")
+        if self.cgroups.version != 1:
+            self.skipTest("not relevant in unified hierarchy")
+        (result, output) = self.execute_run(self.cat, "/proc/self/cgroup")
+        self.check_exitcode(result, 0, "exit code of cat is not zero")
         for line in output:
             if re.match(r"^[0-9]*:([^:]*,)?cpu(,[^:]*)?:/(.*/)?benchmark_.*$", line):
                 return  # Success
         self.fail("Not in expected cgroup for subsystem cpu:\n" + "\n".join(output))
 
     def test_set_cgroup_cpu_shares(self):
-        if not os.path.exists("/bin/echo"):
-            self.skipTest("missing /bin/echo")
+        if not os.path.exists(self.echo):
+            self.skipTest("missing echo")
         try:
-            self.setUp(additional_cgroup_subsystems=["cpu"])
+            if self.cgroups.version == 1:
+                self.setUp(additional_cgroup_subsystems=["cpu"])
+            else:
+                self.setUp(additional_cgroup_subsystems=["memory"])
         except SystemExit as e:
             self.skipTest(e)
-        (result, _) = self.execute_run(
-            "/bin/echo", cgroupValues={("cpu", "shares"): 42}
-        )
-        self.check_exitcode(result, 0, "exit code of /bin/echo is not zero")
+        if self.cgroups.version == 1:
+            cgValues = {("cpu", "shares"): 42}
+        else:
+            cgValues = {("memory", "high"): 420000000}
+        (result, _) = self.execute_run(self.echo, cgroupValues=cgValues)
+        self.check_exitcode(result, 0, "exit code of echo is not zero")
         # Just assert that execution was successful,
         # testing that the value was actually set is much more difficult.
 
     def test_nested_runexec(self):
-        if not os.path.exists("/bin/echo"):
-            self.skipTest("missing /bin/echo")
+        if not os.path.exists(self.echo):
+            self.skipTest("missing echo")
         self.setUp(
             dir_modes={
                 # Do not mark /home hidden, would fail with python from virtualenv
@@ -760,7 +801,7 @@ class TestRunExecutor(unittest.TestCase):
                 "/sys/fs/cgroup": containerexecutor.DIR_FULL_ACCESS,
             }
         )
-        inner_args = ["--", "/bin/echo", "TEST_TOKEN"]
+        inner_args = ["--", self.echo, "TEST_TOKEN"]
 
         with tempfile.NamedTemporaryFile(
             mode="r", prefix="inner_output_", suffix=".log"
@@ -775,16 +816,16 @@ class TestRunExecutor(unittest.TestCase):
         logging.info("Inner output:\n%s", "\n".join(inner_output))
         self.check_result_keys(outer_result, "returnvalue")
         self.check_exitcode(outer_result, 0, "exit code of inner runexec is not zero")
-        self.check_command_in_output(inner_output, "/bin/echo TEST_TOKEN")
+        self.check_command_in_output(inner_output, f"{self.echo} TEST_TOKEN")
         self.assertEqual(
             inner_output[-1], "TEST_TOKEN", "run output misses command output"
         )
 
     def test_starttime(self):
-        if not os.path.exists("/bin/echo"):
-            self.skipTest("missing /bin/echo")
+        if not os.path.exists(self.echo):
+            self.skipTest("missing echo")
         before = util.read_local_time()
-        (result, _) = self.execute_run("/bin/echo")
+        (result, _) = self.execute_run(self.echo)
         after = util.read_local_time()
         self.check_result_keys(result)
         run_starttime = result["starttime"]
@@ -794,9 +835,9 @@ class TestRunExecutor(unittest.TestCase):
 
     def test_frozen_process(self):
         # https://github.com/sosy-lab/benchexec/issues/840
-        if not os.path.exists("/bin/sleep"):
-            self.skipTest("missing /bin/sleep")
-        if not os.path.exists("/sys/fs/cgroup/freezer"):
+        if not os.path.exists(self.sleep):
+            self.skipTest("missing sleep")
+        if self.cgroups.version == 1 and not os.path.exists("/sys/fs/cgroup/freezer"):
             self.skipTest("missing freezer cgroup")
         self.setUp(
             dir_modes={
@@ -806,15 +847,13 @@ class TestRunExecutor(unittest.TestCase):
                 "/sys/fs/cgroup": containerexecutor.DIR_FULL_ACCESS,
             }
         )
-        (result, output) = self.execute_run(
-            "/bin/sh",
-            "-c",
-            """#!/bin/sh
+        script_v1 = """#!/bin/sh
 # create process, move it to sub-cgroup, and freeze it
 set -eu
 
 cgroup="/sys/fs/cgroup/freezer/$(grep freezer /proc/self/cgroup | cut -f 3 -d :)"
 mkdir "$cgroup/tmp"
+mkdir "$cgroup/tmp/tmp"
 
 sleep 10 &
 child_pid=$!
@@ -825,9 +864,37 @@ echo FROZEN > "$cgroup/tmp/freezer.state"
 chmod 000 "$cgroup/tmp/freezer.state"
 chmod 000 "$cgroup/tmp/tasks"
 chmod 000 "$cgroup/tmp"
+chmod 000 "$cgroup/freezer.state"
+chmod 000 "$cgroup/tasks"
 echo FROZEN
 wait $child_pid
-""",
+"""
+        script_v2 = """#!/bin/sh
+# create process, move it to sub-cgroup, and freeze it
+set -eu
+
+cgroup="/sys/fs/cgroup/$(cut -f 3 -d : /proc/self/cgroup)"
+mkdir "$cgroup/tmp"
+mkdir "$cgroup/tmp/tmp"
+
+sleep 10 &
+child_pid=$!
+
+echo $child_pid > "$cgroup/tmp/cgroup.procs"
+echo 1 > "$cgroup/tmp/cgroup.freeze"
+# remove permissions in order to test our handling of this case
+chmod 000 "$cgroup/tmp/cgroup.freeze"
+chmod 000 "$cgroup/tmp/cgroup.procs"
+chmod 000 "$cgroup/tmp"
+chmod 000 "$cgroup/cgroup.freeze"
+chmod 000 "$cgroup/cgroup.kill"
+echo FROZEN
+wait $child_pid
+"""
+        (result, output) = self.execute_run(
+            "/bin/sh",
+            "-c",
+            script_v1 if self.cgroups.version == 1 else script_v2,
             walltimelimit=1,
             expect_terminationreason="walltime",
         )
@@ -1114,10 +1181,32 @@ class TestRunExecutorWithContainer(TestRunExecutor):
         finally:
             shutil.rmtree(temp_dir)
 
+    def test_cpuinfo_with_lxcfs(self):
+        if not os.path.exists("/var/lib/lxcfs/proc"):
+            self.skipTest("missing lxcfs")
+        result, output = self.execute_run(
+            self.grep, "^processor", "/proc/cpuinfo", cores=[0]
+        )
+        self.check_result_keys(result)
+        self.check_exitcode(result, 0, "exit code for reading cpuinfo is not zero")
+        cpus = [int(line.split()[2]) for line in output if line.startswith("processor")]
+        self.assertListEqual(cpus, [0], "Unexpected CPU cores visible in container")
+
+    def test_sys_cpu_with_lxcfs(self):
+        if not os.path.exists("/var/lib/lxcfs/proc"):
+            self.skipTest("missing lxcfs")
+        result, output = self.execute_run(
+            self.cat, "/sys/devices/system/cpu/online", cores=[0]
+        )
+        self.check_result_keys(result)
+        self.check_exitcode(result, 0, "exit code for reading online CPUs is not zero")
+        cpus = util.parse_int_list(output[-1])
+        self.assertListEqual(cpus, [0], "Unexpected CPU cores online in container")
+
     def test_uptime_with_lxcfs(self):
         if not os.path.exists("/var/lib/lxcfs/proc"):
             self.skipTest("missing lxcfs")
-        result, output = self.execute_run("cat", "/proc/uptime")
+        result, output = self.execute_run(self.cat, "/proc/uptime")
         self.check_result_keys(result)
         self.check_exitcode(result, 0, "exit code for reading uptime is not zero")
         uptime = float(output[-1].split(" ")[0])
@@ -1130,7 +1219,7 @@ class TestRunExecutorWithContainer(TestRunExecutor):
             self.skipTest("missing lxcfs")
         # create RunExecutor with desired parameter
         self.setUp(container_system_config=False)
-        result, output = self.execute_run("cat", "/proc/uptime")
+        result, output = self.execute_run(self.cat, "/proc/uptime")
         self.check_result_keys(result)
         self.check_exitcode(result, 0, "exit code for reading uptime is not zero")
         uptime = float(output[-1].split(" ")[0])
@@ -1139,6 +1228,135 @@ class TestRunExecutorWithContainer(TestRunExecutor):
             uptime, 10, f"Uptime {uptime}s unexpectedly low in container"
         )
 
+    def test_fuse_overlay(self):
+        if not container.get_fuse_overlayfs_executable():
+            self.skipTest("fuse-overlayfs not available")
+        with tempfile.TemporaryDirectory(prefix="BenchExec_test_") as temp_dir:
+            test_file_path = os.path.join(temp_dir, "test_file")
+            with open(test_file_path, "wb") as test_file:
+                test_file.write(b"TEST_TOKEN")
+
+            self.setUp(
+                dir_modes={
+                    "/": containerexecutor.DIR_READ_ONLY,
+                    "/home": containerexecutor.DIR_HIDDEN,
+                    "/tmp": containerexecutor.DIR_HIDDEN,
+                    temp_dir: containerexecutor.DIR_OVERLAY,
+                },
+            )
+            result, output = self.execute_run(
+                "/bin/sh",
+                "-c",
+                f"if [ $({self.cat} {test_file_path}) != TEST_TOKEN ]; then exit 1; fi; \
+                {self.echo} TOKEN_CHANGED >{test_file_path}",
+            )
+            self.check_result_keys(result, "returnvalue")
+            self.check_exitcode(result, 0, "exit code of inner runexec is not zero")
+            self.assertTrue(
+                os.path.exists(test_file_path),
+                f"File '{test_file_path}' removed, output was:\n" + "\n".join(output),
+            )
+            with open(test_file_path, "rb") as test_file:
+                test_token = test_file.read()
+            self.assertEqual(
+                test_token.strip(),
+                b"TEST_TOKEN",
+                f"File '{test_file_path}' content is incorrect. Expected 'TEST_TOKEN', but got:\n{test_token}",
+            )
+
+    def test_triple_nested_runexec(self):
+        if not container.get_fuse_overlayfs_executable():
+            self.skipTest("missing fuse-overlayfs")
+
+        # Check if COV_CORE_SOURCE environment variable is set and remove it.
+        # This is necessary because the coverage tool will not work in the nested runexec.
+        coverage_env_var = os.environ.pop("COV_CORE_SOURCE", None)
+
+        with tempfile.TemporaryDirectory(prefix="BenchExec_test_") as temp_dir:
+            overlay_dir = os.path.join(temp_dir, "overlay")
+            os.makedirs(overlay_dir)
+            test_file = os.path.join(overlay_dir, "TEST_FILE")
+            output_dir = os.path.join(temp_dir, "output")
+            os.makedirs(output_dir)
+            mid_output_file = os.path.join(output_dir, "mid_output.log")
+            inner_output_file = os.path.join(output_dir, "inner_output.log")
+            with open(test_file, "w") as f:
+                f.write("TEST_TOKEN")
+                f.seek(0)
+
+            outer_cmd = [
+                "python3",
+                runexec,
+                "--full-access-dir",
+                "/",
+                "--overlay-dir",
+                overlay_dir,
+                "--full-access-dir",
+                output_dir,
+                "--hidden-dir",
+                "/tmp",
+                "--full-access-dir",
+                os.getcwd(),
+                "--output",
+                mid_output_file,
+                "--",
+            ]
+            mid_cmd = [
+                "python3",
+                runexec,
+                "--full-access-dir",
+                "/",
+                "--overlay-dir",
+                overlay_dir,
+                "--full-access-dir",
+                output_dir,
+                "--hidden-dir",
+                "/tmp",
+                "--full-access-dir",
+                os.getcwd(),
+                "--output",
+                inner_output_file,
+                "--",
+            ]
+            inner_cmd = [
+                "/bin/sh",
+                "-c",
+                f"if [ $({self.cat} {test_file}) != TEST_TOKEN ]; then exit 1; fi; {self.echo} TOKEN_CHANGED >{test_file}",
+            ]
+            combined_cmd = outer_cmd + mid_cmd + inner_cmd
+
+            self.setUp(
+                dir_modes={
+                    "/": containerexecutor.DIR_FULL_ACCESS,
+                    "/tmp": containerexecutor.DIR_HIDDEN,
+                    overlay_dir: containerexecutor.DIR_OVERLAY,
+                    output_dir: containerexecutor.DIR_FULL_ACCESS,
+                    os.getcwd(): containerexecutor.DIR_FULL_ACCESS,
+                },
+            )
+            outer_result, outer_output = self.execute_run(*combined_cmd)
+            self.check_result_keys(outer_result, "returnvalue")
+            self.check_exitcode(
+                outer_result, 0, "exit code of outer runexec is not zero"
+            )
+            with open(mid_output_file, "r") as f:
+                self.assertIn("returnvalue=0", f.read().strip().splitlines())
+            self.assertTrue(
+                os.path.exists(test_file),
+                f"File '{test_file}' removed, output was:\n" + "\n".join(outer_output),
+            )
+            with open(test_file, "r") as f:
+                test_token = f.read()
+                self.assertEqual(
+                    test_token.strip(),
+                    "TEST_TOKEN",
+                    f"File '{test_file}' content is incorrect. Expected 'TEST_TOKEN', but got:\n{test_token}",
+                )
+
+        # Restore COV_CORE_SOURCE environment variable
+        if coverage_env_var is not None:
+            os.environ["COV_CORE_SOURCE"] = coverage_env_var
+
 
 class _StopRunThread(threading.Thread):
     def __init__(self, delay, runexecutor):
