@@ -6,18 +6,22 @@
 # SPDX-License-Identifier: Apache-2.0
 
 import logging
-import sys
 import os
 import re
+import sys
 
 import benchexec.result as result
 import benchexec.tools.template
+from benchexec.tools.sv_benchmarks_util import (
+    get_non_witness_input_files_or_identifier,
+    get_witness_options,
+)
+from benchexec.tools.template import ToolNotFoundException
 
 
 class Tool(benchexec.tools.template.BaseTool2):
     """
     Tool info for CPAchecker, the Configurable Software-Verification Platform.
-    URL: https://cpachecker.sosy-lab.org/
 
     Both binary and source distributions of CPAchecker are supported.
     If the source of CPAchecker is present, it is checked wether the compiled binaries
@@ -32,6 +36,7 @@ class Tool(benchexec.tools.template.BaseTool2):
     """
 
     REQUIRED_PATHS = [
+        "bin/cpachecker",
         "lib/java/runtime",
         "lib/*.jar",
         "lib/native/x86_64-linux",
@@ -41,24 +46,50 @@ class Tool(benchexec.tools.template.BaseTool2):
     ]
 
     def executable(self, tool_locator):
-        executable = tool_locator.find_executable("cpa.sh", subdir="scripts")
+        # The following will perform these lookups (if --tool-directory is not given)
+        # and pick the first one that is found:
+        # 1. cpachecker in PATH
+        # 2. cpachecker in ./ and bin/
+        # 3. cpa.sh in PATH
+        # 4. cpa.sh in ./ and scripts/
+        # This follows the BenchExec logic of "look up first in PATH, then ./"
+        # except for the case of "cpa.sh in PATH and cpachecker in bin/",
+        # which should be ok.
+        try:
+            executable = tool_locator.find_executable("cpachecker", subdir="bin")
+        except ToolNotFoundException as e1:
+            try:
+                executable = tool_locator.find_executable("cpa.sh", subdir="scripts")
+            except ToolNotFoundException:
+                raise e1
+
         base_dir = os.path.join(os.path.dirname(executable), os.path.pardir)
         jar_file = os.path.join(base_dir, "cpachecker.jar")
-        bin_dir = os.path.join(base_dir, "bin")
+        cls_dir = os.path.join(
+            base_dir, "bin" if executable.endswith("cpa.sh") else "classes"
+        )
         src_dir = os.path.join(base_dir, "src")
 
         # If this is a source checkout of CPAchecker, we heuristically check that
         # sources are not newer than binaries (cpachecker.jar or files in bin/).
-        if os.path.isdir(src_dir):
-            src_mtime = self._find_newest_mtime(src_dir)
+        try:
+            has_jar = os.path.isfile(jar_file)
+            has_cls = os.path.isdir(cls_dir)
 
-            if os.path.isfile(jar_file):
-                if src_mtime > os.stat(jar_file).st_mtime:
-                    sys.exit("CPAchecker JAR is not uptodate, run 'ant jar'!")
+            if os.path.isdir(src_dir) and (has_jar or has_cls):
+                src_mtime = self._find_newest_mtime(src_dir)
 
-            elif os.path.isdir(bin_dir):
-                if src_mtime > self._find_newest_mtime(bin_dir):
-                    sys.exit("CPAchecker build is not uptodate, run 'ant'!")
+                if has_jar:
+                    if src_mtime > os.stat(jar_file).st_mtime:
+                        sys.exit("CPAchecker JAR is not uptodate, run 'ant jar'!")
+
+                elif has_cls:
+                    if src_mtime > self._find_newest_mtime(cls_dir):
+                        sys.exit("CPAchecker build is not uptodate, run 'ant'!")
+        except OSError as e:
+            logging.warning(
+                "Could not determine whether CPAchecker needs to be rebuilt: %s", e
+            )
 
         return executable
 
@@ -79,42 +110,93 @@ class Tool(benchexec.tools.template.BaseTool2):
         version = self._version_from_tool(executable, "-help", line_prefix="CPAchecker")
         return version.split("(")[0].strip()
 
+    def url_for_version(self, version):
+        if ":" in version:
+            # Revision from VerifierCloud WebClient like "trunk:44918"
+            branch, revision = version.split(":", maxsplit=1)
+            if branch != "trunk":
+                branch = f"branches/{branch}"
+            return f"https://svn.sosy-lab.org/trac/cpachecker/browser/CPAchecker/{branch}/?rev={revision}"
+
+        elif re.fullmatch("[0-9.]+", version):
+            # Release version like "2.2.1"
+            return f"https://svn.sosy-lab.org/trac/cpachecker/browser/CPAchecker/tags/cpachecker-{version}/"
+
+        elif re.fullmatch("[0-9.]+-svn [0-9]{1,5}", version):
+            # Old development version like "1.7-svn 20000"
+            # Could end in "M", but then has local changes and we do not want a link.
+            version = version.rsplit(" ", maxsplit=1)[-1]
+            return f"https://svn.sosy-lab.org/trac/cpachecker/browser/CPAchecker/trunk/?rev={version}"
+
+        elif re.fullmatch("[0-9.]+-svn-[0-9]{5}", version):
+            # Recent development version like "2.0-svn-30000"
+            # Could end in "M", but then has local changes and we do not want a link.
+            version = version.rsplit("-", maxsplit=1)[-1]
+            return f"https://svn.sosy-lab.org/trac/cpachecker/browser/CPAchecker/trunk/?rev={version}"
+
+        elif re.fullmatch("[0-9.]+-svn-[0-9a-f]{6,}", version):
+            # Development version with git commit like "2.2.1-svn-9743f6eae7"
+            # Could end in "+", but then has local changes and we do not want a link.
+            version = version.rsplit("-", maxsplit=1)[-1]
+            return f"https://gitlab.com/sosy-lab/software/cpachecker/-/tree/{version}"
+
+        elif re.fullmatch("[0-9.]+-[0-9]+-g[0-9a-f]{6,}", version):
+            # Development version with git commit like "4.0-123-gabcdef"
+            # Could end in "+", but then has local changes and we do not want a link.
+            version = version.rsplit("g", maxsplit=1)[-1]
+            return f"https://gitlab.com/sosy-lab/software/cpachecker/-/tree/{version}"
+
+        elif re.fullmatch("[0-9a-f]{40}", version):
+            # Full git hash produced by VerifierCloud WebClient
+            return f"https://gitlab.com/sosy-lab/software/cpachecker/-/tree/{version}"
+
+        return None
+
     def name(self):
         return "CPAchecker"
 
-    def _get_additional_options(self, existing_options, task, rlimits):
-        options = []
-        if rlimits.cputime and "-timelimit" not in existing_options:
-            options += ["-timelimit", f"{rlimits.cputime}s"]
+    def project_url(self):
+        return "https://cpachecker.sosy-lab.org/"
 
-        if "-stats" not in existing_options:
-            options += ["-stats"]
+    def _get_additional_options(self, existing_options, task, rlimits):
+        # If at least one option uses "--" style, we use that for options added here.
+        # Otherwise use "-" to be safe for old versions of CPAchecker.
+        new_option_style = any(option.startswith("--") for option in existing_options)
+        prefix = "--" if new_option_style else "-"
+
+        def option_present(option):
+            return f"-{option}" in existing_options or f"--{option}" in existing_options
+
+        options = []
+        if rlimits.cputime and not option_present("timelimit"):
+            options += [f"{prefix}timelimit", f"{rlimits.cputime}s"]
+
+        if not option_present("stats"):
+            options += [f"{prefix}stats"]
 
         if task.property_file:
-            options += ["-spec", task.property_file]
+            options += [f"{prefix}spec", task.property_file]
 
         if isinstance(task.options, dict) and task.options.get("language") == "C":
             data_model = task.options.get("data_model")
             if data_model:
-                data_model_option = {"ILP32": "-32", "LP64": "-64"}.get(data_model)
+                data_model_option = {"ILP32": "32", "LP64": "64"}.get(data_model)
                 if data_model_option:
-                    if data_model_option not in existing_options:
-                        options += [data_model_option]
+                    if not option_present(data_model_option):
+                        options += [f"{prefix}{data_model_option}"]
                 else:
                     raise benchexec.tools.template.UnsupportedFeatureException(
                         f"Unsupported data_model '{data_model}' defined for task '{task}'"
                     )
 
+        options += get_witness_options(existing_options, task, [f"{prefix}witness"])
+
         return options
 
     def cmdline(self, executable, options, task, rlimits):
         additional_options = self._get_additional_options(options, task, rlimits)
-        return (
-            [executable]
-            + options
-            + additional_options
-            + list(task.input_files_or_identifier)
-        )
+        input_files = get_non_witness_input_files_or_identifier(task)
+        return [executable] + options + additional_options + input_files
 
     def determine_result(self, run):
         """
