@@ -25,7 +25,7 @@ class Tool(benchexec.tools.template.BaseTool):
             elif line.startswith("1 safe, 0 unsafe"):
                 status = result.RESULT_TRUE_PROP
             elif ((returnsignal == 9) or (returnsignal == 15)) and isTimeout:
-                status = "TIMEOUT"
+                status = result.RESULT_TIMEOUT
             elif returnsignal == 9:
                 status = "KILLED BY SIGNAL 9"
 
