@@ -13,7 +13,7 @@ import benchexec.result as result
 
 class Tool(benchexec.tools.template.BaseTool2):
     """
-    This class serves as tool adaptor for FuSeBMC (https://github.com/kaled-alshmrany/FuSeBMC)
+    This class serves as tool adaptor for FuSeBMC
     """
 
     def executable(self, tool_locator):
@@ -29,6 +29,9 @@ class Tool(benchexec.tools.template.BaseTool2):
     def name(self):
         return "FuSeBMC"
 
+    def project_url(self):
+        return "https://github.com/kaled-alshmrany/FuSeBMC"
+
     def cmdline(self, executable, options, task, rlimits):
         if task.property_file:
             options = options + ["-p", task.property_file]
