@@ -12,7 +12,7 @@ import benchexec.result as result
 
 class Tool(benchexec.tools.template.BaseTool):
     """
-    Wrapper for a PAGAI tool (http://pagai.forge.imag.fr/).
+    Wrapper for a PAGAI tool.
     """
 
     def executable(self):
@@ -21,6 +21,9 @@ class Tool(benchexec.tools.template.BaseTool):
     def name(self):
         return "PAGAI"
 
+    def project_url(self):
+        return "http://pagai.forge.imag.fr/"
+
     def determine_result(self, returncode, returnsignal, output, isTimeout):
         output = "\n".join(output)
         if ((returnsignal == 9) or (returnsignal == 15)) and isTimeout:
