@@ -16,7 +16,6 @@ class Smtlib2Tool(benchexec.tools.template.BaseTool):
     """
 
     def determine_result(self, returncode, returnsignal, output, isTimeout):
-
         if returnsignal == 0 and returncode == 0:
             status = None
             for line in output:
@@ -32,7 +31,7 @@ class Smtlib2Tool(benchexec.tools.template.BaseTool):
                 status = result.RESULT_UNKNOWN
 
         elif ((returnsignal == 9) or (returnsignal == 15)) and isTimeout:
-            status = "TIMEOUT"
+            status = result.RESULT_TIMEOUT
 
         elif returnsignal == 9:
             status = "KILLED BY SIGNAL 9"
