@@ -5,9 +5,15 @@
 #
 # SPDX-License-Identifier: Apache-2.0
 
-from benchexec.tools.sv_benchmarks_util import get_data_model_from_task, ILP32, LP64
 import benchexec.tools.template
 import benchexec.result as result
+from benchexec.tools.sv_benchmarks_util import (
+    get_data_model_from_task,
+    ILP32,
+    LP64,
+    TaskFilesConsidered,
+    handle_witness_of_task,
+)
 
 
 class Tool(benchexec.tools.template.BaseTool2):
@@ -50,7 +56,14 @@ class Tool(benchexec.tools.template.BaseTool2):
         if data_model_param and data_model_param not in options:
             options += [data_model_param]
 
-        return [executable] + options + list(task.input_files_or_identifier)
+        input_files, witness_options = handle_witness_of_task(
+            task,
+            options,
+            "--witness-check",
+            TaskFilesConsidered.INPUT_FILES_OR_IDENTIFIER,
+        )
+
+        return [executable] + options + witness_options + input_files
 
     def determine_result(self, run):
         if run.was_timeout:
