@@ -5,6 +5,7 @@
 #
 # SPDX-License-Identifier: Apache-2.0
 
+import benchexec.result as result
 import benchexec.util as util
 import benchexec.tools.template
 
@@ -20,7 +21,7 @@ class Tool(benchexec.tools.template.BaseTool):
         output = "\n".join(output)
         if returnsignal == 9 or returnsignal == (128 + 9):
             if isTimeout:
-                status = "TIMEOUT"
+                status = result.RESULT_TIMEOUT
             else:
                 status = "KILLED BY SIGNAL 9"
         elif returncode == 1 and "program correct: ERROR unreachable" in output:
