@@ -77,6 +77,12 @@ class VcloudBenchmarkBase(benchexec.benchexec.BenchExec):
             action="store_true",
             help="Allows the usage of cgroups inside the execution environment. This is useful e.g. if a tool wants to make use of resource limits for subprocesses it spawns.",
         )
+        vcloud_args.add_argument(
+            "--tryLessMemory",
+            dest="tryLessMemory",
+            action="store_true",
+            help="Execute runs first with less memory than specified. In case a run fails because of OOM, it is automatically rescheduled but this time with full memory limit.",
+        )
         vcloud_args.add_argument(
             self.get_param_name("cloudAdditionalFiles"),
             dest="additional_files",
@@ -91,6 +97,14 @@ class VcloudBenchmarkBase(benchexec.benchexec.BenchExec):
             action="store_true",
             help="Prevents ivy from caching the downloaded jar files. This prevents clashes due to concurrent access to the cache.",
         )
+        vcloud_args.add_argument(
+            self.get_param_name("cloudContainerImage"),
+            dest="containerImage",
+            metavar="IMAGE",
+            action="store",
+            type=str,
+            help="Use the specified container image for the execution of the benchmark.",
+        )
 
     def get_param_name(self, pname):
         return "--v" + pname
